@@ -14,7 +14,7 @@
 #include <cstring>
 #include <filesystem>
 #include <memory>
-#include <ranges>
+#include <queue>
 #include <span>
 
 #include <QCoreApplication>
@@ -50,6 +50,16 @@
 using namespace KIO;
 using namespace std::filesystem;
 
+template<typename Output, typename Input>
+constexpr Output narrow(Input i)
+{
+    Output o = static_cast<Input>(i);
+    if (i != Input(o)) {
+        std::abort();
+    }
+    return o;
+}
+
 namespace std
 {
 template<>
@@ -61,6 +71,16 @@ struct default_delete<struct sftp_file_struct> {
         }
     }
 };
+
+#if defined(HAVE_SFTP_AIO)
+template<>
+struct default_delete<struct sftp_aio_struct> {
+    void operator()(struct sftp_aio_struct *ptr) const
+    {
+        sftp_aio_free(ptr);
+    }
+};
+#endif
 } // namespace std
 
 using UniqueSFTPFilePtr = std::unique_ptr<struct sftp_file_struct>;
@@ -68,6 +88,7 @@ using UniqueSFTPFilePtr = std::unique_ptr<struct sftp_file_struct>;
 namespace
 {
 constexpr auto KIO_SFTP_SPECIAL_TIMEOUT_MS = 30;
+constexpr auto MAX_PENDING_REQUESTS = 128;
 
 // How big should each data packet be? Definitely not bigger than 64kb or
 // you will overflow the 2 byte size variable in a sftp packet.
@@ -191,9 +212,6 @@ void log_callback(int priority, const char *function, const char *buffer, void *
 
 inline std::optional<perms> posixToOptionalPerms(uint32_t mode) noexcept
 {
-    if (mode < 0) {
-        return {};
-    }
     return static_cast<perms>(mode) & perms::mask;
 }
 
@@ -423,7 +441,7 @@ int SFTPWorker::authenticateKeyboardInteractive(AuthInfo &info)
     return err;
 }
 
-Result SFTPWorker::reportError(const QUrl &url, const int err)
+Result SFTPWorker::reportError(const QUrl &url, int err)
 {
     qCDebug(KIO_SFTP_LOG) << "url = " << url << " - err=" << err;
 
@@ -488,13 +506,15 @@ Result SFTPWorker::createUDSEntry(SFTPAttributesPtr sb, UDSEntry &entry, const Q
         case SSH_FILEXFER_TYPE_UNKNOWN:
             fileType = QT_STAT_MASK - 1;
             break;
+        default: // type is an unsigned int and may contain anything, explicitly default to break
+            break;
         }
         access = posixToOptionalPerms(sb->permissions).value_or(perms::none);
         size = sb->size;
     }
     entry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, fileType);
     entry.fastInsert(KIO::UDSEntry::UDS_ACCESS, permsToPosix(access));
-    entry.fastInsert(KIO::UDSEntry::UDS_SIZE, size);
+    entry.fastInsert(KIO::UDSEntry::UDS_SIZE, narrow<long long>(size));
 
     if (details > 0) {
         if (sb->owner) {
@@ -515,7 +535,7 @@ Result SFTPWorker::createUDSEntry(SFTPAttributesPtr sb, UDSEntry &entry, const Q
         if (sb->flags & SSH_FILEXFER_ATTR_CREATETIME) {
             // Availability depends on outside factors.
             // https://bugs.kde.org/show_bug.cgi?id=375305
-            entry.fastInsert(KIO::UDSEntry::UDS_CREATION_TIME, sb->createtime);
+            entry.fastInsert(KIO::UDSEntry::UDS_CREATION_TIME, narrow<long long>(sb->createtime));
         }
     }
 
@@ -742,8 +762,7 @@ Result SFTPWorker::openConnectionWithoutCloseOnError()
     // Start the ssh connection.
 
     // Attempt to start a ssh session and establish a connection with the server.
-    const Result openResult = sftpOpenConnection(info);
-    if (!openResult.success()) {
+    if (auto openResult = sftpOpenConnection(info); !openResult.success()) {
         return openResult;
     }
 
@@ -1040,8 +1059,7 @@ Result SFTPWorker::open(const QUrl &url, QIODevice::OpenMode mode)
 {
     qCDebug(KIO_SFTP_LOG) << "open: " << url;
 
-    const auto loginResult = sftpLogin();
-    if (!loginResult.success()) {
+    if (auto loginResult = sftpLogin(); !loginResult.success()) {
         return loginResult;
     }
 
@@ -1061,6 +1079,7 @@ Result SFTPWorker::open(const QUrl &url, QIODevice::OpenMode mode)
         return Result::fail(KIO::ERR_CANNOT_OPEN_FOR_READING, url.toDisplayString());
     case SSH_FILEXFER_TYPE_SYMLINK:
     case SSH_FILEXFER_TYPE_REGULAR:
+    default: // type is an unsigned int and may contain anything, explicitly default to break
         break;
     }
 
@@ -1120,7 +1139,7 @@ Result SFTPWorker::read(KIO::filesize_t bytes)
 
     Q_ASSERT(mOpenFile != nullptr);
 
-    QVarLengthArray<char> buffer(bytes);
+    QVarLengthArray<char> buffer(narrow<qsizetype>(bytes));
 
     ssize_t bytesRead = sftp_read(mOpenFile, buffer.data(), bytes);
     Q_ASSERT(bytesRead <= static_cast<ssize_t>(bytes));
@@ -1248,8 +1267,7 @@ Result SFTPWorker::sftpGet(const QUrl &url, KIO::fileoffset_t offset, int fd)
 {
     qCDebug(KIO_SFTP_LOG) << url;
 
-    const auto loginResult = sftpLogin();
-    if (!loginResult.success()) {
+    if (auto loginResult = sftpLogin(); !loginResult.success()) {
         return loginResult;
     }
 
@@ -1270,6 +1288,7 @@ Result SFTPWorker::sftpGet(const QUrl &url, KIO::fileoffset_t offset, int fd)
         return Result::fail(KIO::ERR_CANNOT_OPEN_FOR_READING, url.toString());
     case SSH_FILEXFER_TYPE_SYMLINK:
     case SSH_FILEXFER_TYPE_REGULAR:
+    default: // type is an unsigned int and may contain anything, explicitly default to break
         break;
     }
 
@@ -1348,8 +1367,7 @@ Result SFTPWorker::sftpPut(const QUrl &url, int permissionsMode, JobFlags flags,
 
     auto permissions(posixToOptionalPerms(permissionsMode));
 
-    const auto loginResult = sftpLogin();
-    if (!loginResult.success()) {
+    if (auto loginResult = sftpLogin(); !loginResult.success()) {
         return loginResult;
     }
 
@@ -1386,7 +1404,7 @@ Result SFTPWorker::sftpPut(const QUrl &url, int permissionsMode, JobFlags flags,
                 qCDebug(KIO_SFTP_LOG) << "put got answer " << (flags & KIO::Resume);
 
             } else {
-                KIO::filesize_t pos = QT_LSEEK(fd, sbPart->size, SEEK_SET);
+                KIO::filesize_t pos = QT_LSEEK(fd, narrow<off_t>(sbPart->size), SEEK_SET);
                 if (pos != sbPart->size) {
                     qCDebug(KIO_SFTP_LOG) << "Failed to seek to" << sbPart->size << "bytes in source file. Reason given:" << strerror(errno);
                     return Result::fail(ERR_CANNOT_SEEK, url.toString());
@@ -1418,13 +1436,13 @@ Result SFTPWorker::sftpPut(const QUrl &url, int permissionsMode, JobFlags flags,
     UniqueSFTPFilePtr destFile{};
     KIO::fileoffset_t totalBytesSent = 0;
     if ((flags & KIO::Resume)) {
-        qCDebug(KIO_SFTP_LOG) << "Trying to append: " << dest;
+        qCDebug(KIO_SFTP_LOG) << "Trying to open for append: " << dest;
         destFile.reset(sftp_open(mSftp, dest.constData(), O_RDWR, 0)); // append if resuming
         if (destFile) {
             SFTPAttributesPtr fstat(sftp_fstat(destFile.get()));
             if (fstat) {
                 sftp_seek64(destFile.get(), fstat->size); // Seek to end TODO
-                totalBytesSent += fstat->size;
+                totalBytesSent += narrow<KIO::fileoffset_t>(fstat->size);
             }
         }
     } else {
@@ -1465,7 +1483,7 @@ Result SFTPWorker::sftpPut(const QUrl &url, int permissionsMode, JobFlags flags,
     } // file
 
     auto reader = [fd, this]() -> QCoro::Generator<ReadResponse> {
-        for (int result = 1; result > 0;) {
+        for (ssize_t result = 1; result > 0;) {
             ReadResponse response;
             if (fd == -1) {
                 dataReq(); // Request for data
@@ -1480,7 +1498,7 @@ Result SFTPWorker::sftpPut(const QUrl &url, int permissionsMode, JobFlags flags,
                     qCDebug(KIO_SFTP_LOG) << "failed to read" << errno;
                     response.error = ERR_CANNOT_READ;
                 } else {
-                    response.filedata = QByteArray(buf.data(), result);
+                    response.filedata = QByteArray(buf.data(), narrow<qsizetype>(result));
                 }
             }
 
@@ -1505,7 +1523,7 @@ Result SFTPWorker::sftpPut(const QUrl &url, int permissionsMode, JobFlags flags,
             return closeOnError(KIO::ERR_CANNOT_WRITE);
         }
 
-        totalBytesSent += response.bytes;
+        totalBytesSent += narrow<decltype(totalBytesSent)>(response.bytes);
         processedSize(totalBytesSent);
     }
 
@@ -1644,7 +1662,7 @@ Result SFTPWorker::sftpCopyGet(const QUrl &url, const QString &sCopyFile, int pe
     int fd = -1;
     KIO::fileoffset_t offset = 0;
     if (bResume) {
-        fd = QT_OPEN(QFile::encodeName(sPart).constData(), O_RDWR); // append if resuming
+        fd = QT_OPEN(QFile::encodeName(sPart).constData(), O_RDWR | O_CLOEXEC); // append if resuming
         offset = QT_LSEEK(fd, partFile.size(), SEEK_SET);
         if (offset != partFile.size()) {
             qCDebug(KIO_SFTP_LOG) << "Failed to seek to" << partFile.size() << "bytes in target file. Reason given:" << strerror(errno);
@@ -1653,7 +1671,7 @@ Result SFTPWorker::sftpCopyGet(const QUrl &url, const QString &sCopyFile, int pe
         }
         qCDebug(KIO_SFTP_LOG) << "resuming at" << offset;
     } else {
-        fd = QT_OPEN(QFile::encodeName(dest).constData(), O_CREAT | O_TRUNC | O_WRONLY, permsToPosix(initialMode));
+        fd = QT_OPEN(QFile::encodeName(dest).constData(), O_CREAT | O_TRUNC | O_WRONLY | O_CLOEXEC, permsToPosix(initialMode));
     }
 
     if (fd == -1) {
@@ -1727,7 +1745,7 @@ Result SFTPWorker::sftpCopyPut(const QUrl &url, const QString &sCopyFile, int pe
         return Result::fail(ERR_DOES_NOT_EXIST, sCopyFile);
     }
 
-    const int fd = QT_OPEN(QFile::encodeName(sCopyFile).constData(), O_RDONLY);
+    const int fd = QT_OPEN(QFile::encodeName(sCopyFile).constData(), O_RDONLY | O_CLOEXEC);
     if (fd == -1) {
         return Result::fail(ERR_CANNOT_OPEN_FOR_READING, sCopyFile);
     }
@@ -1744,8 +1762,7 @@ Result SFTPWorker::stat(const QUrl &url)
 {
     qCDebug(KIO_SFTP_LOG) << url;
 
-    const auto loginResult = sftpLogin();
-    if (!loginResult.success()) {
+    if (auto loginResult = sftpLogin(); !loginResult.success()) {
         return loginResult;
     }
 
@@ -1781,7 +1798,7 @@ Result SFTPWorker::stat(const QUrl &url)
     }
 
     UDSEntry entry;
-    const Result result = createUDSEntry(SFTPAttributesPtr(attributes), entry, path, QFileInfo(path).fileName(), details);
+    auto result = createUDSEntry(SFTPAttributesPtr(attributes), entry, path, QFileInfo(path).fileName(), details);
     if (!result.success()) {
         return result;
     }
@@ -1794,8 +1811,7 @@ Result SFTPWorker::mimetype(const QUrl &url)
 {
     qCDebug(KIO_SFTP_LOG) << url;
 
-    const auto loginResult = sftpLogin();
-    if (!loginResult.success()) {
+    if (auto loginResult = sftpLogin(); !loginResult.success()) {
         return loginResult;
     }
 
@@ -1810,8 +1826,7 @@ Result SFTPWorker::listDir(const QUrl &url)
 {
     qCDebug(KIO_SFTP_LOG) << "list directory: " << url;
 
-    const auto loginResult = sftpLogin();
-    if (!loginResult.success()) {
+    if (auto loginResult = sftpLogin(); !loginResult.success()) {
         return loginResult;
     }
 
@@ -1874,8 +1889,7 @@ Result SFTPWorker::mkdir(const QUrl &url, int permissions)
 {
     qCDebug(KIO_SFTP_LOG) << "create directory: " << url;
 
-    const auto loginResult = sftpLogin();
-    if (!loginResult.success()) {
+    if (auto loginResult = sftpLogin(); !loginResult.success()) {
         return loginResult;
     }
 
@@ -1894,7 +1908,7 @@ Result SFTPWorker::mkdir(const QUrl &url, int permissions)
     qCDebug(KIO_SFTP_LOG) << "Trying to create directory: " << path;
     SFTPAttributesPtr sb(sftp_lstat(mSftp, path_c.constData()));
     if (sb == nullptr) {
-        if (sftp_mkdir(mSftp, path_c.constData(), 0777) < 0) {
+        if (sftp_mkdir(mSftp, path_c.constData(), S_IRWXU | S_IRWXG | S_IRWXO) < 0) {
             return reportError(url, sftp_get_error(mSftp));
         }
 
@@ -1917,8 +1931,7 @@ Result SFTPWorker::rename(const QUrl &src, const QUrl &dest, KIO::JobFlags flags
 {
     qCDebug(KIO_SFTP_LOG) << "rename " << src << " to " << dest << flags;
 
-    const auto loginResult = sftpLogin();
-    if (!loginResult.success()) {
+    if (auto loginResult = sftpLogin(); !loginResult.success()) {
         return loginResult;
     }
 
@@ -1955,8 +1968,7 @@ Result SFTPWorker::symlink(const QString &target, const QUrl &dest, KIO::JobFlag
 {
     qCDebug(KIO_SFTP_LOG) << "link " << target << "->" << dest << ", overwrite = " << (flags & KIO::Overwrite) << ", resume = " << (flags & KIO::Resume);
 
-    const auto loginResult = sftpLogin();
-    if (!loginResult.success()) {
+    if (auto loginResult = sftpLogin(); !loginResult.success()) {
         return loginResult;
     }
 
@@ -1992,8 +2004,7 @@ Result SFTPWorker::chmod(const QUrl &url, int permissions)
 {
     qCDebug(KIO_SFTP_LOG) << "change permission of " << url << " to " << QString::number(permissions);
 
-    const auto loginResult = sftpLogin();
-    if (!loginResult.success()) {
+    if (auto loginResult = sftpLogin(); !loginResult.success()) {
         return loginResult;
     }
 
@@ -2010,8 +2021,7 @@ Result SFTPWorker::del(const QUrl &url, bool isfile)
 {
     qCDebug(KIO_SFTP_LOG) << "deleting " << (isfile ? "file: " : "directory: ") << url;
 
-    const auto loginResult = sftpLogin();
-    if (!loginResult.success()) {
+    if (auto loginResult = sftpLogin(); !loginResult.success()) {
         return loginResult;
     }
 
@@ -2036,6 +2046,214 @@ void SFTPWorker::worker_status()
     workerStatus((mConnected ? mHost : QString()), mConnected);
 }
 
+void SFTPWorker::requiresUserNameRedirection()
+{
+    QUrl redirectUrl;
+    redirectUrl.setScheme(QLatin1String("sftp"));
+    redirectUrl.setUserName(mUsername);
+    redirectUrl.setPassword(mPassword);
+    redirectUrl.setHost(mHost);
+    if (mPort > 0 && mPort != DEFAULT_SFTP_PORT) {
+        redirectUrl.setPort(mPort);
+    }
+    qCDebug(KIO_SFTP_LOG) << "redirecting to" << redirectUrl;
+    redirection(redirectUrl);
+}
+
+Result SFTPWorker::sftpLogin()
+{
+    const QString origUsername = mUsername;
+    if (auto openResult = openConnection(); !openResult.success()) {
+        return openResult;
+    }
+    qCDebug(KIO_SFTP_LOG) << "connected ?" << mConnected << "username: old=" << origUsername << "new=" << mUsername;
+    if (!origUsername.isEmpty() && origUsername != mUsername) {
+        requiresUserNameRedirection();
+        return Result::fail();
+    }
+    return mConnected ? Result::pass() : Result::fail();
+}
+
+void SFTPWorker::clearPubKeyAuthInfo()
+{
+    if (mPublicKeyAuthInfo) {
+        delete mPublicKeyAuthInfo;
+        mPublicKeyAuthInfo = nullptr;
+    }
+}
+
+Result SFTPWorker::fileSystemFreeSpace(const QUrl &url)
+{
+    qCDebug(KIO_SFTP_LOG) << "file system free space of" << url;
+
+    if (auto loginResult = sftpLogin(); !loginResult.success()) {
+        return loginResult;
+    }
+
+    if (sftp_extension_supported(mSftp, "statvfs@openssh.com", "2") == 0) {
+        return Result::fail(ERR_UNSUPPORTED_ACTION, QString());
+    }
+
+    const QByteArray path = url.path().isEmpty() ? QByteArrayLiteral("/") : url.path().toUtf8();
+
+    sftp_statvfs_t statvfs = sftp_statvfs(mSftp, path.constData());
+    if (statvfs == nullptr) {
+        return reportError(url, sftp_get_error(mSftp));
+    }
+
+    setMetaData(QString::fromLatin1("total"), QString::number(statvfs->f_frsize * statvfs->f_blocks));
+    setMetaData(QString::fromLatin1("available"), QString::number(statvfs->f_frsize * statvfs->f_bavail));
+
+    sftp_statvfs_free(statvfs);
+
+    return Result::pass();
+}
+
+#if defined(HAVE_SFTP_AIO)
+using UniqueAIO = std::unique_ptr<struct sftp_aio_struct>;
+
+QCoro::Generator<SFTPWorker::ReadResponse> SFTPWorker::asyncRead(sftp_file file, size_t size)
+{
+    size_t queuedBytes = 0;
+    std::queue<UniqueAIO> pendingRequests;
+
+    auto queueChunkMaybe = [&pendingRequests, &queuedBytes, size, file]() -> int {
+        if (queuedBytes >= size) {
+            return KJob::NoError;
+        }
+
+        const auto requestLength = std::min<int>(MAX_XFER_BUF_SIZE, size - queuedBytes);
+        sftp_aio aio = nullptr;
+        if (sftp_aio_begin_read(file, requestLength, &aio) == SSH_ERROR) {
+            qCWarning(KIO_SFTP_LOG) << "Failed to sftp_aio_begin_read" //
+                                    << "- SFTP error:" << sftp_get_error(file->sftp) //
+                                    << "- SSH error:" << ssh_get_error_code(file->sftp->session) //
+                                    << "- SSH errorString:" << ssh_get_error(file->sftp->session);
+            return KIO::ERR_CANNOT_READ;
+        }
+
+        pendingRequests.emplace(aio);
+        queuedBytes += requestLength;
+        return KJob::NoError;
+    };
+
+    // Queue a bunch of requests
+    while (pendingRequests.size() < MAX_PENDING_REQUESTS && queuedBytes < size) {
+        if (auto error = queueChunkMaybe(); error != KJob::NoError) {
+            // Cleanup of pending requests happens through queue destruction
+            co_yield ReadResponse(error);
+            co_return;
+        }
+    }
+
+    // pop-read-queue_new-yield loop until all requests are processed
+    size_t receivedBytes = 0;
+    std::array<char, MAX_XFER_BUF_SIZE> buffer{};
+    std::span bufferSpan{buffer};
+    while (!pendingRequests.empty()) {
+        auto aio = pendingRequests.front().release();
+        pendingRequests.pop();
+        if (auto error = queueChunkMaybe(); error != KJob::NoError) {
+            // Cleanup of pending requests happens through queue destruction
+            co_yield ReadResponse(error);
+            co_return;
+        }
+
+        ssize_t readBytes = 0;
+        while (true) {
+            auto readSpan = bufferSpan.subspan(readBytes);
+            readBytes = sftp_aio_wait_read(&aio, readSpan.data(), readSpan.size());
+            if (readBytes == SSH_AGAIN) {
+                continue;
+            }
+            if (readBytes == SSH_ERROR) {
+                qCWarning(KIO_SFTP_TRACE_LOG) << "read SSH_ERROR";
+                co_yield ReadResponse(KIO::ERR_CANNOT_READ);
+                co_return;
+            }
+
+            receivedBytes += readBytes;
+            if (readBytes != MAX_XFER_BUF_SIZE && receivedBytes != size) { // short read
+                qCWarning(KIO_SFTP_TRACE_LOG) << "unexpected short read. the file probably was truncated";
+                co_yield ReadResponse(KIO::ERR_CANNOT_READ);
+                co_return;
+            }
+
+            co_yield ReadResponse(QByteArray(buffer.data(), readBytes));
+            break;
+        }
+    }
+}
+
+QCoro::Generator<SFTPWorker::WriteResponse> SFTPWorker::asyncWrite(sftp_file file, QCoro::Generator<ReadResponse> reader)
+{
+    std::queue<UniqueAIO> pendingRequests;
+
+    auto readIt = reader.begin();
+    auto readEnd = reader.end();
+    auto queueChunkMaybe = [file, &pendingRequests, &readIt, &readEnd]() -> int {
+        if (readIt == readEnd) {
+            return KJob::NoError;
+        }
+
+        const auto &readResponse = *readIt;
+        if (readResponse.error != KJob::NoError) {
+            return readResponse.error;
+        }
+
+        const auto requestLength = readResponse.filedata.size();
+        sftp_aio aio = nullptr;
+        if (sftp_aio_begin_write(file, readResponse.filedata.constData(), requestLength, &aio) == SSH_ERROR) {
+            qCWarning(KIO_SFTP_LOG) << "Failed to sftp_aio_begin_write" //
+                                    << "- SFTP error:" << sftp_get_error(file->sftp) //
+                                    << "- SSH error:" << ssh_get_error_code(file->sftp->session) //
+                                    << "- SSH errorString:" << ssh_get_error(file->sftp->session);
+            return KIO::ERR_CANNOT_READ;
+        }
+
+        pendingRequests.emplace(aio);
+        ++readIt;
+        return KJob::NoError;
+    };
+
+    // Queue a bunch of requests
+    while (pendingRequests.size() < MAX_PENDING_REQUESTS && readIt != reader.end()) {
+        if (auto error = queueChunkMaybe(); error != KJob::NoError) {
+            // Cleanup of pending requests happens through queue destruction
+            co_yield {.error = error};
+            co_return;
+        }
+    }
+
+    while (!pendingRequests.empty()) {
+        auto aio = pendingRequests.front().release();
+        pendingRequests.pop();
+        if (auto error = queueChunkMaybe(); error != KJob::NoError) {
+            // Cleanup of pending requests happens through queue destruction
+            co_yield {.error = error};
+            co_return;
+        }
+
+        ssize_t writtenBytes = 0;
+        while (true) {
+            writtenBytes = sftp_aio_wait_write(&aio);
+            if (writtenBytes == SSH_AGAIN) {
+                continue;
+            }
+            if (writtenBytes == SSH_ERROR) {
+                qCWarning(KIO_SFTP_TRACE_LOG) << "write SSH_ERROR";
+                co_yield {.error = KIO::ERR_CANNOT_WRITE};
+                co_return;
+            }
+
+            co_yield {.bytes = std::make_unsigned_t<size_t>(writtenBytes)};
+            break;
+        }
+    }
+}
+
+#else
+
 SFTPWorker::GetRequest::GetRequest(sftp_file file, uint64_t size, ushort maxPendingRequests)
     : m_file(file)
     , m_size(size)
@@ -2075,15 +2293,15 @@ bool SFTPWorker::GetRequest::enqueueChunks()
     return true;
 }
 
-int SFTPWorker::GetRequest::readChunks(QByteArray &data)
+size_t SFTPWorker::GetRequest::readChunks(QByteArray &data)
 {
-    int totalRead = 0;
+    size_t totalRead = 0;
     ssize_t bytesread = 0;
     const uint64_t initialOffset = m_file->offset;
 
     while (!m_pendingRequests.isEmpty()) {
         SFTPWorker::GetRequest::Request &request = m_pendingRequests.head();
-        int dataSize = data.size() + request.expectedLength;
+        auto dataSize = data.size() + request.expectedLength;
 
         data.resize(dataSize);
         if (data.size() < dataSize) {
@@ -2167,71 +2385,6 @@ SFTPWorker::GetRequest::~GetRequest()
     }
 }
 
-void SFTPWorker::requiresUserNameRedirection()
-{
-    QUrl redirectUrl;
-    redirectUrl.setScheme(QLatin1String("sftp"));
-    redirectUrl.setUserName(mUsername);
-    redirectUrl.setPassword(mPassword);
-    redirectUrl.setHost(mHost);
-    if (mPort > 0 && mPort != DEFAULT_SFTP_PORT) {
-        redirectUrl.setPort(mPort);
-    }
-    qCDebug(KIO_SFTP_LOG) << "redirecting to" << redirectUrl;
-    redirection(redirectUrl);
-}
-
-Result SFTPWorker::sftpLogin()
-{
-    const QString origUsername = mUsername;
-    const auto openResult = openConnection();
-    if (!openResult.success()) {
-        return openResult;
-    }
-    qCDebug(KIO_SFTP_LOG) << "connected ?" << mConnected << "username: old=" << origUsername << "new=" << mUsername;
-    if (!origUsername.isEmpty() && origUsername != mUsername) {
-        requiresUserNameRedirection();
-        return Result::fail();
-    }
-    return mConnected ? Result::pass() : Result::fail();
-}
-
-void SFTPWorker::clearPubKeyAuthInfo()
-{
-    if (mPublicKeyAuthInfo) {
-        delete mPublicKeyAuthInfo;
-        mPublicKeyAuthInfo = nullptr;
-    }
-}
-
-Result SFTPWorker::fileSystemFreeSpace(const QUrl &url)
-{
-    qCDebug(KIO_SFTP_LOG) << "file system free space of" << url;
-
-    const auto loginResult = sftpLogin();
-    if (!loginResult.success()) {
-        return loginResult;
-    }
-
-    if (sftp_extension_supported(mSftp, "statvfs@openssh.com", "2") == 0) {
-        return Result::fail(ERR_UNSUPPORTED_ACTION, QString());
-    }
-
-    const QByteArray path = url.path().isEmpty() ? QByteArrayLiteral("/") : url.path().toUtf8();
-
-    sftp_statvfs_t statvfs = sftp_statvfs(mSftp, path.constData());
-    if (statvfs == nullptr) {
-        return reportError(url, sftp_get_error(mSftp));
-    }
-
-    setMetaData(QString::fromLatin1("total"), QString::number(statvfs->f_frsize * statvfs->f_blocks));
-    setMetaData(QString::fromLatin1("available"), QString::number(statvfs->f_frsize * statvfs->f_bavail));
-
-    sftp_statvfs_free(statvfs);
-
-    return Result::pass();
-}
-
 QCoro::Generator<SFTPWorker::ReadResponse> SFTPWorker::asyncRead(sftp_file file, size_t size)
 {
     SFTPWorker::GetRequest request(file, size);
@@ -2294,5 +2447,6 @@ QCoro::Generator<SFTPWorker::WriteResponse> SFTPWorker::asyncWrite(sftp_file fil
         }
     }
 }
+#endif // HAVE_SFTP_AIO
 
 #include "kio_sftp.moc"
